@@ -41,20 +41,18 @@ from __future__ import unicode_literals
 
 import codecs
 import copy
+import difflib
 import fnmatch
 import inspect
+import io
+import locale
+import optparse
 import os
 import re
 import signal
 import sys
-try:
-    from StringIO import StringIO
-except ImportError:
-    from io import StringIO
 import token
 import tokenize
-from optparse import OptionParser
-import difflib
 
 import pep8
 
@@ -65,7 +63,7 @@ except NameError:
     unicode = str
 
 
-__version__ = '0.9.2'
+__version__ = '0.9.3'
 
 
 CR = '\r'
@@ -85,7 +83,7 @@ SHORTEN_OPERATOR_GROUPS = frozenset([
 ])
 
 
-DEFAULT_IGNORE = 'E24,W6'
+DEFAULT_IGNORE = 'E24'
 
 
 def open_with_encoding(filename, encoding=None, mode='r'):
@@ -93,7 +91,6 @@ def open_with_encoding(filename, encoding=None, mode='r'):
     if not encoding:
         encoding = detect_encoding(filename)
 
-    import io
     return io.open(filename, mode=mode, encoding=encoding,
                    newline='')  # Preserve line endings
 
@@ -187,7 +184,7 @@ class FixPEP8(object):
         if contents is None:
             self.source = read_from_filename(filename, readlines=True)
         else:
-            sio = StringIO(contents)
+            sio = io.StringIO(contents)
             self.source = sio.readlines()
         self.newline = find_newline(self.source)
         self.options = options
@@ -230,6 +227,10 @@ class FixPEP8(object):
                 line_index = result['line'] - 1
                 original_line = self.source[line_index]
 
+                # TODO: Make below logical fix stuff more efficient. Can we
+                # salvage things such that we can still apply logical fixes
+                # even though there are modified lines?
+
                 is_logical_fix = len(inspect.getargspec(fix).args) > 2
                 if is_logical_fix:
                     # Do not run logical fix if any lines have been modified.
@@ -291,6 +292,12 @@ class FixPEP8(object):
             print('--->  {n} issue(s) to fix {progress}'.format(
                 n=len(results), progress=progress), file=sys.stderr)
 
+        if self.options.line_range:
+            results = [
+                r for r in results
+                if self.options.line_range[0] <= r['line'] <=
+                self.options.line_range[1]]
+
         self._fix_source(filter_results(source=''.join(self.source),
                                         results=results,
                                         aggressive=self.options.aggressive))
@@ -311,16 +318,14 @@ class FixPEP8(object):
         logical_start = []
         logical_end = []
         last_newline = True
-        sio = StringIO(''.join(self.source))
+        sio = io.StringIO(''.join(self.source))
         parens = 0
         for t in tokenize.generate_tokens(sio.readline):
             if t[0] in [tokenize.COMMENT, tokenize.DEDENT,
                         tokenize.INDENT, tokenize.NL,
                         tokenize.ENDMARKER]:
                 continue
-            if not parens and t[0] in [
-                tokenize.NEWLINE, tokenize.SEMI
-            ]:
+            if not parens and t[0] in [tokenize.NEWLINE, tokenize.SEMI]:
                 last_newline = True
                 logical_end.append((t[3][0] - 1, t[2][1]))
                 continue
@@ -439,14 +444,7 @@ class FixPEP8(object):
     def fix_e125(self, result, logical):
         """Indent to distinguish line from next logical line."""
         # Fix by indenting the line in error to the next stop.
-        modified_lines = self._fix_reindent(result, logical)
-        if modified_lines:
-            return modified_lines
-        else:
-            # Fallback
-            line_index = result['line'] - 1
-            original_line = self.source[line_index]
-            self.source[line_index] = self.indent_word + original_line
+        return self._fix_reindent(result, logical)
 
     def fix_e126(self, result, logical):
         """Fix over-indented hanging indentation."""
@@ -712,7 +710,7 @@ class FixPEP8(object):
         indent = _get_indentation(target)
         source = target[len(indent):]
         assert source.lstrip() == source
-        sio = StringIO(source)
+        sio = io.StringIO(source)
 
         # Check for multiline string.
         try:
@@ -728,6 +726,18 @@ class FixPEP8(object):
             else:
                 return []
 
+        # Handle statements by putting the right hand side on a line by itself.
+        # This should let the next pass shorten it.
+        if self.options.aggressive and source.startswith('return '):
+            self.source[line_index] = (
+                indent +
+                'return (' +
+                self.newline +
+                indent + self.indent_word + re.sub('^return ', '', source) +
+                indent + ')' + self.newline
+            )
+            return
+
         candidates = shorten_line(
             tokens, source, indent,
             self.indent_word, newline=self.newline,
@@ -874,7 +884,7 @@ class FixPEP8(object):
         return range(1, 1 + original_length)
 
 
-def fix_e26(source):
+def fix_e26(source, _=False):
     """Format block comments."""
     if '#' not in source:
         # Optimization.
@@ -885,7 +895,7 @@ def fix_e26(source):
         include_docstrings=True) | set(commented_out_code_lines(source))
 
     fixed_lines = []
-    sio = StringIO(source)
+    sio = io.StringIO(source)
     for (line_number, line) in enumerate(sio.readlines(), start=1):
         if (line.lstrip().startswith('#') and
                 line_number not in ignored_line_numbers):
@@ -929,14 +939,20 @@ def refactor(source, fixer_names, ignore=None):
     return new_text
 
 
-def fix_w602(source):
+def fix_w602(source, aggressive=True):
     """Fix deprecated form of raising exception."""
+    if not aggressive:
+        return source
+
     return refactor(source, ['raise'],
                     ignore='with_traceback')
 
 
-def fix_w6(source):
+def fix_w6(source, aggressive=True):
     """Fix various deprecated code (via lib2to3)."""
+    if not aggressive:
+        return source
+
     return refactor(source,
                     ['apply',
                      'except',
@@ -944,7 +960,6 @@ def fix_w6(source):
                      'has_key',
                      'idioms',
                      'import',
-                     'methodattrs',  # Python >= 2.6
                      'ne',
                      'numliterals',
                      'operator',
@@ -981,7 +996,7 @@ def find_newline(source):
 
 def _get_indentword(source):
     """Return indentation type."""
-    sio = StringIO(source)
+    sio = io.StringIO(source)
     indent_word = '    '  # Default in case source has no indentation
     try:
         for t in tokenize.generate_tokens(sio.readline):
@@ -1455,7 +1470,7 @@ class Wrapper(object):
         self.lines = physical_lines
         self.tokens = []
         self.rel_indent = None
-        sio = StringIO(''.join(physical_lines))
+        sio = io.StringIO(''.join(physical_lines))
         for t in tokenize.generate_tokens(sio.readline):
             if not len(self.tokens) and t[0] in self.SKIP_TOKENS:
                 continue
@@ -1761,9 +1776,7 @@ def filter_results(source, results, aggressive=False):
         issue_id = r['id'].lower()
 
         if r['line'] in non_docstring_string_line_numbers:
-            if issue_id.startswith('e1'):
-                continue
-            elif issue_id in ['e501', 'w191']:
+            if issue_id.startswith(('e1', 'e501', 'w191')):
                 continue
 
         if r['line'] in all_string_line_numbers:
@@ -1775,7 +1788,7 @@ def filter_results(source, results, aggressive=False):
         if not aggressive and (r['line'] + 1) in all_string_line_numbers:
             # Do not modify multiline strings in non-aggressive mode. Remove
             # trailing whitespace could break doctests.
-            if issue_id.startswith('w29') or issue_id.startswith('w39'):
+            if issue_id.startswith(('w29', 'w39')):
                 continue
 
         # Filter out incorrect E101 reports when there are no tabs.
@@ -1784,11 +1797,12 @@ def filter_results(source, results, aggressive=False):
         if issue_id == 'e101' and '\t' not in split_source[r['line']]:
             continue
 
-        if issue_id in ['e711', 'e712'] and not aggressive:
-            continue
+        if not aggressive:
+            if issue_id.startswith(('e711', 'e712', 'w6')):
+                continue
 
         if r['line'] in commented_out_code_line_numbers:
-            if issue_id == 'e501' or issue_id.startswith('e26'):
+            if issue_id.startswith(('e26', 'e501')):
                 continue
 
         yield r
@@ -1802,7 +1816,7 @@ def multiline_string_lines(source, include_docstrings=False):
     Docstrings are ignored.
 
     """
-    sio = StringIO(source)
+    sio = io.StringIO(source)
     line_numbers = set()
     previous_token_type = ''
     try:
@@ -1834,7 +1848,7 @@ def commented_out_code_lines(source):
     clutter.
 
     """
-    sio = StringIO(source)
+    sio = io.StringIO(source)
     line_numbers = []
     try:
         for t in tokenize.generate_tokens(sio.readline):
@@ -1923,7 +1937,12 @@ def fix_string(source, options=None):
     if not options:
         options = parse_args([''])[0]
 
-    sio = StringIO(source)
+    try:
+        source = source.decode(locale.getpreferredencoding(False))
+    except AttributeError:
+        pass
+
+    sio = io.StringIO(source)
     return fix_lines(sio.readlines(), options=options)
 
 
@@ -1934,8 +1953,11 @@ def fix_lines(source_lines, options, filename=''):
     # Keep a history to break out of cycles.
     previous_hashes = set([hash(tmp_source)])
 
-    # Apply global fixes only once (for efficiency).
-    fixed_source = apply_global_fixes(tmp_source, options)
+    if options.line_range:
+        fixed_source = tmp_source
+    else:
+        # Apply global fixes only once (for efficiency).
+        fixed_source = apply_global_fixes(tmp_source, options)
 
     passes = 0
     while True:
@@ -1977,7 +1999,7 @@ def fix_file(filename, options=None, output=None):
     fixed_source = fix_lines(fixed_source, options, filename=filename)
 
     if options.diff:
-        new = StringIO(fixed_source)
+        new = io.StringIO(fixed_source)
         new = new.readlines()
         diff = get_diff_text(original_source, new, filename)
         if output:
@@ -2003,7 +2025,7 @@ def global_fixes():
     for function in globals().values():
         if inspect.isfunction(function):
             arguments = inspect.getargspec(function)[0]
-            if arguments != ['source']:
+            if arguments[:1] != ['source']:
                 continue
 
             code = extract_code_from_function(function)
@@ -2023,7 +2045,7 @@ def apply_global_fixes(source, options):
             if options.verbose:
                 print('--->  Applying global fix for {0}'.format(code.upper()),
                       file=sys.stderr)
-            source = function(source)
+            source = function(source, options.aggressive)
 
     return source
 
@@ -2047,12 +2069,12 @@ def extract_code_from_function(function):
 
 def parse_args(args):
     """Parse command-line options."""
-    parser = OptionParser(usage='Usage: %prog [options] '
-                                '[filename [filename ...]]'
-                                '\nUse filename \'-\'  for stdin.',
-                          version='%prog {0}'.format(__version__),
-                          description=__doc__.split('\n')[0],
-                          prog='autopep8')
+    parser = optparse.OptionParser(usage='Usage: %prog [options] '
+                                         '[filename [filename ...]]'
+                                         '\nUse filename \'-\'  for stdin.',
+                                   version='%prog {0}'.format(__version__),
+                                   description=__doc__.split('\n')[0],
+                                   prog='autopep8')
     parser.add_option('-v', '--verbose', action='count', dest='verbose',
                       default=0,
                       help='print verbose messages; '
@@ -2088,6 +2110,11 @@ def parse_args(args):
     parser.add_option('--max-line-length', metavar='n', default=79, type=int,
                       help='set maximum allowed line length '
                            '(default: %default)')
+    parser.add_option('--range', metavar='start end', dest='line_range',
+                      default=None, type=int, nargs=2,
+                      help='only fix errors found within this inclusive '
+                           'range of line numbers (e.g. 1 99); '
+                           'line numbers are indexed at 1')
     options, args = parser.parse_args(args)
 
     if not len(args) and not options.list_fixes:
@@ -2373,8 +2400,7 @@ def main():
 
             # LineEndingWrapper is unnecessary here due to the symmetry between
             # standard in and standard out.
-            sys.stdout.write(fix_string(sys.stdin.read(),
-                                        options))
+            sys.stdout.write(fix_string(sys.stdin.read(), options))
         else:
             if options.in_place or options.diff:
                 filenames = list(set(args))
